function [Omega] = transform_KSVD_learn_omega(Y,M,N,L,iter,qq)
% Learn a sparsifying transform (operator) from data using Transform K-SVD.
% Realizes the Transform K-SVD algorithm from the paper:
% K-SVD meets Transform Learning: Transform K-SVD, 
% E.M. Eksioglu and Ozden Bayir, IEEE Signal Processing Letters, accepted for publication, 2014. 
% Y = training data
% M = Signal dimension
% N = Transform domain dimension
% L = Number of Training Samples
% qq= sparsity for thresholding
% iter = Number of Iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adapted from the Analysis K-SVD realization available on:
% http://www.cs.technion.ac.il/~elad/software/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Initialize the transform operator
 Omega=zeros(N,M);
    for j=1:N
        Omega(j,:)=GenerateRandomRow(Y);
    end
    alpha=1000;
%% Alternating minimisation using  the transform update and coefficients update.
for k = 1:iter
        k
  %% X update using hardthreshlding
    X=Omega*Y;
    % hard threshold
       XX=sort(abs(X),'descend');
       ttt=XX(qq+1,:);
       ttt1=repmat(ttt,N,1);
       X=X.*(abs(X)>ttt1);  
       % soft threshold
% X = X - min(sthresh,abs(X)).*sign(X);
 %% Omega (or W) update  
       for jj=1:L
       [CoS(:,jj)] = abs(X(:,jj))<1e-9;
       end
    NumSigs = sum(CoS,2);
    [NumSigss,IdxOmega]=sort(NumSigs,'descend');
     OmegaPrev=Omega;
     CoEx=double(CoS)*(1-double(CoS))';
        Omega=Omega(IdxOmega,:);
        CoS=CoS(IdxOmega,:);
        CoEx=CoEx(IdxOmega,IdxOmega);
        E=eye(N);
     
        for j=1:N
            Ys=Y(:,CoS(j,:));  % the signals orthogonal to the current row
            CovY=Ys*Ys';
           
            Idx1=find(E(j,:)==0);
                OmegaCW=repmat(sqrt(CoEx(Idx1,j)),1,M).*Omega(Idx1,:);
                RegTerm=alpha*(OmegaCW'*OmegaCW); % encourage orthogonality
                [v,lambdaaa]=eigs(CovY+RegTerm,1,'sm',struct('disp',0));
                Omega(j,:)=v';
        end
 Omega=normrows(Omega);
   
  if k<iter+1
        % Find the rows which did not update in the last iteration
        cond = abs(diag(OmegaPrev*Omega'))>0.999;        
    for j = 1:N
        if (any(abs(Omega(1:j-1,:)*Omega(j,:)') > 0.95)) || cond(j)
            %%If row already appeared or does not update as it should - replace it
            Omega(j,:)=GenerateRandomRow(Y);
            fprintf('Replaced row no. %d - already appeared or does not update as it should\n', j);
           Omega(j,:)=normrows(Omega(j,:));
        end
    end
  end
end

Omega=normrows(Omega);

